<?php

//phpinfo();

// tparser.php
// script to parse trustee /et /eo save all [filepath] file output
// to identify longish paths and owners, and the email a notifiction

/*
 University of Illinois/NCSA Open Source License for tparser.php.

 Copyright (c) 2009 Jim Champ/Surrey County Council
 All rights reserved.
 Developed by: 		Jim Champ
 www.devboats.co.uk/idmphp/
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the
 "Software"), to deal with the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimers.

 Redistributions in binary form must reproduce the above
 copyright notice, this list of conditions and the following disclaimers
 in the documentation and/or other materials provided with the
 distribution.

 Neither the names of Surrey County Council nor the names of its contributors may be used to endorse
 or promote products derived from this Software without specific prior
 written permission.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.

 tparser.php

 */

// Control Variables

// for some situations, esp debugging you may need to change the standard timeout for PHP scripts
set_time_limit(300);

// Warnings are generated for all documents that exceed this path length.
// Its probably best to have this at around 220, which is the limit at which some windows apps bomb out
// although I believe the documented Win limit is 216
// However if introducing it for the first time it may be worth starting with a much higher limit
// in order to reduce the number of warnings sent out and thus the load on support desk.
$warningCount = 220;

// set true for the output(s) as required...
$create_email_output = false;
$create_html_file_output = true;

// I've found you want the full path on the web page, but YMMV
// If you set this to true you'll get the mappings happening on the web page too... 
// As theweb page is probably for administrators to use, this seems to be inconvenient because
// the admin probably won't have the same drive mappings as the users so will
// have to translate back to find the files
$UseMappingInHTMLOutput = true;

// File names and Paths...

// This is the path where the output from trustee and the drive mappings file are stored
// note that you may need extra \\ characters as  it is of course the
// escape character in php eg "\\\chf2\vol1\\" for "\\chf2\vol1 or "vol1:\data\exports\\"
// UNC won't work i you are running this from a netware server
// But this app will run equally well from a Win workstation with a Novell client
// and a user with admin rights.
// null is script directory of course 
$srcDataPath = "";
// this is the output of trustee.nlm et eo
$trusteeOutputFile = "example.txt";

// and this the file used to store the table of conversions of server path to user path eg
// "VOL1:Data",H:\DATA"
// "VOL2:Data\shared\departments\widgets",G:\widgets"
// Note that this is case sensitive - you need to check the output from trustee before
// creating the file to get the paths right.
// Exactly where you map this depends on your drive mappings in login scripts, map root commands
// and so on. It may take you some trial and error to get this right
// note that paths shouldn't finish with a trailing \ thisis a "feature"!
$pathMappingTableFname = "mappings.txt";

// This is the location an filename where the output html file is placed
// null is script directory of course
$htmlOutputPath = "";
$htmlOutputFname = "longfiles.htm";

// Various EMail Addresses
$mail_From = "support@yourdomain.com";
// Mail is sent to default if there is no email address found for the owner in LDAP - this could
// be users who no longer exist or systems ids with no email address set.
$defaultOwnerEmail = "server.team@yourdomain.com";
// This is a friendly name for messages sent to above
$defaultOwnerFriendlyName = "Administrators";
// these placeholders are used in formatting the mail message...

//LDAP ACcount Settings. This is the account used to look up email addresses
$ldap_pw = "";
$ldap_server = "ldap.yourdomain.com";
$ldap_root = "";
$ldap_id = "cn=admin,o=yourdomain" ;

// and set the SMTP server (for netware and Windows at least)
ini_set("SMTP","mail.yourdomain.com" ); 

//Now the message format. Note the use of the placeholders, which will be
// replaced with the user details in the processing loop
// There can be some subtle tuning of the mail message in the loop by
// adding extra text alongside the placeholder

$mail_Title = "Warning: You have documents that may not be accessible.";

$mail_Header = "Dear " . $friendlyNamePlaceHolder . ",\n\n";
$mail_Header .= "";
$mail_Header .= "Warning: The system lists " . $userNamePlaceHolder;
$mail_Header .= " as owning/having created documents that may not be able to be accessed by Windows.";
$mail_Header .= "The following list of documents have a total system path (folder and file name) length in excess of ";
$mail_Header .= $warningCount . " characters. This can cause problems reading and writing files from Windows.\n";
$mail_Header .= "You are advised to shorten folder and file names so that the total length is not more than ";
$mail_Header .= "about 200 characters. It's especially worth looking at folder names as this problem ";
$mail_Header .= "often occurs when folder names are changed or folders are dragged and dropped into other folders.\n";
$mail_Header .= "If any of the documents are redundant and can be deleted then please remove them. ";
$mail_Header .= "You may well have problems removing the files if you do not shorten file and folder names ";
$mail_Header .= "before attempting to delete them. If this occurs shorten names and try again.\n\n";
$mail_Header .= "This problem not only affects the ability to access the files, but may also ";
$mail_Header .= "cause problems backing up the data, so please resolve the problem within the next few days. ";
$mail_Header .= "If you are not responsible for or have no access to some or all of these documents and folders ";
$mail_Header .= "please will you forward this email onto someone who does have access so that the problem ";
$mail_Header .= "can be resolved.\n\n";

$mail_Footer = "\nIf you have any questions or require any help with this please contact the Service desk.\n\n";
$mail_Footer .= "best regards, Technical Support.\n";

$html_Text = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n\"http://www.w3.org/TR/html4/strict.dtd\"><html>\r\n<head>\r\n</head>\r\n<body>\r\n";
$html_Text .= "<h1>Long File Path Report</h1>\r\n";
$html_Text .= "<p>The following list of documents have a total path (folder and file name) length in excess of ";
$html_Text .=  $warningCount . " characters. This can cause problems reading and writing files from Windows.</p>\r\n";
$html_Text .= "<table border='1'>\r\n";

// That ends the configuration

//best not to change these...
$friendlyNamePlaceHolder = "[[insert friendly name here!]]";
$userNamePlaceHolder = "[[insert user name here]]";


// OK lets start with some screen output...

echo "Tparser.php\n";
echo "===========\n";
echo "\nScript to parse output from trustee /et /eo and report on overlength paths.\n";

// Concatenate src file names and paths... There is no sanity checking here, better get them right!
$trusteeOutputFile = $srcDataPath.$trusteeOutputFile;
$pathMappingTableFname = $srcDataPath.$pathMappingTableFname;

// format the mail headers
$mail_Headers  = "From: " .  $mail_From . "\r\n" . "Reply-To: " . $mail_From . "\r\n" . "X-Mailer: PHP/" . phpversion();

// define some other variables
$LastOwner = "";
$ownerEmail = "";
$mail_Message = "";

// OK here we want to read in the drive mappings array
// the idea is that each server has a csv file which contains all the drive mappings for the various folders...
// this depends on mappings being unique of course - ie that the same path isn't accessed by different drive
// letters for the same path or path subset.
// so this feature will only be useful if your server management is fairly disciplined
echo "Reading drive mapping substitution table from " . $pathMappingTableFname . "\n";
$dFileHandle = @fopen($pathMappingTableFname,"r");
$pathMappingArrCtr = 0;
if ($dFileHandle) {
	while (!feof($dFileHandle)) {
		$dBuffer = fgetcsv($dFileHandle,4096,",");
		$pathMappingArray[$pathMappingArrCtr] = $dBuffer;
		// echo $pathMappingArray[$pathMappingArrCtr][0] . "->" . $pathMappingArray[$pathMappingArrCtr][1] . "\n";
		++$pathMappingArrCtr;
	}
} else {
	echo "Could not open the Mapping file\n";
}

//and now read in the trustee output...
echo "Reading file & owners list from " . $trusteeOutputFile . "\n";
$iFileHandle = @fopen ($trusteeOutputFile,"r");
$ExceptionsArrayCtr = 0;

// $screenDisplayCtr and $screenDisplayLineCtr are only used for screen output...
// the idea is that after every $screenDisplayIntervalCtr lines are read then the screen is updated
// the lower the value then the more frequent the updates but the slower the program runs since
// screen output is very slow. Conversely if the number is too large updates the screen won't change enough
// to give the required confidence something is happening...
$screenDisplayCtr = 0;
$screenDisplayLineCtr = 0;
$screenDisplayIntervalCtr = 5000;

if ($iFileHandle) {
	while (!feof($iFileHandle)) {
		// This next is purely to give some screen output for confidence whilst the file
		// is being read.
		if ($screenDisplayCtr > $screenDisplayIntervalCtr){
			$screenDisplayCtr = 0;
			++$screenDisplayLineCtr;
			echo "\rReading line " . $screenDisplayLineCtr * $screenDisplayIntervalCtr ;
		}
		++$screenDisplayCtr;
		// end screen confidence section, can be commented out if unwelcome...

		// read in a line of csv, with trustee.nlm output only 1st and 3rd fields are required
		// but for simplicity in coding stuff the lot in...
		$iBuffer = fgetcsv($iFileHandle,4096,",");
		// and then check the path length (in [1] against our test value
		if(strlen(@$iBuffer[1])> $warningCount) {
			//echo strlen($iBuffer[1]);
			//echo "\t";
			//echo $iBuffer[1];
			//echo "\n";
			//echo $iBuffer[3];
			//echo "\n";
			// and put it in $ExceptionsArray
			$ExceptionsArray[$ExceptionsArrayCtr] = $iBuffer;
			++$ExceptionsArrayCtr;
		}
	}
} else {
	echo "Could not open the trustee output file\n";
}
echo "\nRead Completed.\n";

if ($ExceptionsArrayCtr < 1) {
	// bomb out if no exceptions are found!!
	echo "No overlength Paths detected! Ending Run.\n";
	exit;
}

// at this point $ExceptionsArray contains details of all files > $warningCount...
// we want to sort the array by user name and then by file path..
// this is a bit tricky as multisort sorts by column, not row...
echo "Sorting data by user id.\n";
foreach($ExceptionsArray as $key => $row){
	$col1[$key] = $row[1];
	$col3[$key] = $row[3];
}
// damned if I know understand this crazy syntax, but its out of the book!
// php array handling doesn't quite fit my mind set!
array_multisort($col3,SORT_ASC,SORT_STRING,
				$col1,SORT_ASC,SORT_STRING,
				$ExceptionsArray);

// and here's the sorted array...
// the dump could be uncommented for debugging...
// var_dump($ExceptionsArray);

if($create_email_output){
	// we are going to want to LDAP email addresses , so lets open up the LDAP server...
	echo "Connecting to LDAP server...\n";
	$ds=@ldap_connect("$ldap_server");  // must be a valid LDAP server!
	//echo "Post Connect\n";
	if(@ldap_bind($ds, $ldap_id,$ldap_pw)) {
		echo "Connection succeeded.\n";
		// we have now made an LDAP connection
	} else {
		echo "LDAP Authentication Failed\n";
		echo "User id and/or Password credentials were incorrect.\n";
		exit;
	}
	//echo "Post Bind\n";
}

// now this is all well and good, but what we want to do now is the report
// basically each user gets an email, with a list of all files, and
// horrible warnings etc
// so we want to go through $ExceptionsArray creating text, and each time user changes we fire off another email...

// this first record value is simply to avoid creating null messages...
// this should be something very unique!!


$FirstRecord =  true;

echo "Creating Output...\n";

foreach ($ExceptionsArray as $ExceptionsArrayItem) {
	// at this point we kick in the drive mapping stuff...
	// this is completely crude and nasty, because its case sensitive.
	// that isn't a complete disaster because the trustee output owner
	// should be consistent case wise, but there's definitely potential
	// and I don't want just to do a change case on the whole string...
	// if using php 5 then use str_ireplace instead
	// the code for doing a case insensitive search would be rather complex...
	

	$owner = $ExceptionsArrayItem[3];
	//echo $ExceptionsArrayItem[1];
	//echo "\t";
	//echo $ExceptionsArrayItem[3];
	//echo "\n";
	if($owner == $LastOwner) {
		// Next record, same owner
	} else {
		// store the new ownerfor comparison next time
		$LastOwner = $owner;

		// html output start new owner, 
		if($create_html_file_output) {
			if (!$FirstRecord) {
				// this is just a row separator, but we don't want it above the first row... 
				$html_Text .= "<tr><td></td><td colspan='2'>&nbsp;</td></tr>";
			}
			// and now the header for each new user. Not creating a new table each time - slow
			// display but neater...
			$html_Text .= "<tr><th>Owner</th><th>Path&nbsp;Length</th><th>File Path</th></tr>";
			$html_Text .= "<tr><td colspan='3'>" . $owner . "</td></tr>";
		}
		
		// we are starting a new mail message: Lets mail off the old one.
		if($create_email_output){
			if (!$FirstRecord) {
					
				// here finish off the previous mail message
				// unless this is first time through...
				$mail_Message .= $mail_Footer;
				// and send it off...
				// echo $ownerEmail . " : " . $mail_Title . " : " .    $mail_Headers. " \n" ;
				mail ($ownerEmail,$mail_Title,$mail_Message,$mail_Headers);
			}
			//Now start up a new one...
			// here we want to go LDAP up the new owner email address,
			// close off the previous message and email, start the next message, all that...
			// echo "Running the LDAP Search...\n";
			// do the search
			$ldap_sr = "cn=" . substr($ExceptionsArrayItem[3],0,strpos($ExceptionsArrayItem[3],"."));
			$sr=@ldap_search($ds, $ldap_root, $ldap_sr, array(cn,mail,eMailAddress,surname,givenName) );
			$entry = @ldap_first_entry($ds, $sr);
			$ldap_returns = 0;
			$ldap_returns = @ldap_count_entries($ds, $sr);
			$attrs = @ldap_get_attributes($ds,$entry);
			// echo "Ldap Query\n";
			// We only use the email details from LDAP if there was a single return to the
			// LDAP Query... More or less means we shouldn't trust the adddresses - or even know which one to use!
			if ($ldap_returns = 1) {
				// get the email details in the fields
				$ownerEmail =  @$attrs["mail"][0] ;
				$ownerFriendlyName =  @$attrs["givenName"][0] . " " . @$attrs["surname"][0] ;
			} 
			if (strlen($ownerEmail)> 3)  {
				// we check length of email address - it shoud have returned from LDAP or it could be blank
				
				// and replace the placeholders with the email details in the mail header
				$mail_Message = str_replace($friendlyNamePlaceHolder,$ownerFriendlyName,$mail_Header);
				// with some subtle message changing. Be nice to put this in the stuff at the top
				// I suppose, but really, there are limits!
				$mail_Message = str_replace($userNamePlaceHolder,"your user id, ". $owner,$mail_Message);
			} else {
				
				// if its short we assume we use the default value - a@cc is probably the shortest legal email address!  
				$ownerEmail =  $defaultOwnerEmail;
				// so here we do the replacement slightly differently using the defaults
				$mail_Message = str_replace($friendlyNamePlaceHolder,$defaultOwnerFriendlyName,$mail_Header);
				// and again some extra words...
				$mail_Message = str_replace($userNamePlaceHolder,"user id " . $owner . ", which has no email address listed, ",$mail_Message);
			}
		}
		// A "Something's happening", each new user overwrites the previous onscreen due
		// to cr but not LF
		echo "Processing files for\t" . str_pad($owner,55) . "\r"  ;
	}
	
	//OK, that's all the new/changed user processing done, now we are publishing each row
	// of the main data...


	if($create_html_file_output) {
		//show path lenth...
		$html_Text .= "<tr><td></td><td>" . strlen( $ExceptionsArrayItem[1]) . "</td>";
		if($UseMappingInHTMLOutput){ 
			foreach ($pathMappingArray as $pathMappingArrayItem) {
				$ExceptionsArrayItem[1] = str_replace($pathMappingArrayItem[0],$pathMappingArrayItem[1],$ExceptionsArrayItem[1]);
				//echo $pathMappingArrayItem[0]. "+" . $pathMappingArrayItem[1]. "+" .$ExceptionsArrayItem[1]."\n\n";		
			}
		}
		//and report filename
		$html_Text .= "<td>".  $ExceptionsArrayItem[1] . "</td></tr>\r\n";
	}

	if($create_email_output){
		// report path length
		$mail_Message .= "\t" . strlen( $ExceptionsArrayItem[1]) . " chars.\t";
		// subs drive letters for full path
		foreach ($pathMappingArray as $pathMappingArrayItem) {
			$ExceptionsArrayItem[1] = str_replace($pathMappingArrayItem[0],$pathMappingArrayItem[1],$ExceptionsArrayItem[1]);
			//echo $pathMappingArrayItem[0]. "+" . $pathMappingArrayItem[1]. "+" .$ExceptionsArrayItem[1]."\n\n";		
		}
		// report path
		$mail_Message .=   $ExceptionsArrayItem[1] . "\n";
	}	
	
	
	// and finally turn off our first record catcher... only needs to happen first time of course, but its
	// surely quicker processing to do it every time than to test and then do if required, certainly simpler
	// coding
	$FirstRecord=False;
}

// All is now processed so we can finish up the output...
fclose($iFileHandle);

// now we write out the web page
if($create_html_file_output) {
	$html_Text .= "<tr><td></td><td colspan='2'>&nbsp;</td></tr>";
	$html_Text .= "</table>\nReport Competed.</body>\r\n</html>\r\n";
	@$fHandle = fopen($htmlOutputPath.$htmlOutputFname,"w");
	@fwrite($fHandle,$html_Text);
	fclose($fHandle);
}

//And we mail off the last record
if($create_email_output){
	// here finish off the previous mail message
	$mail_Message .= $mail_Footer;
	$mail_Message .= "\n";
	// and send it off...
	//echo "Sending Last email...\n";
	mail ($ownerEmail,$mail_Title,$mail_Message,$mail_Headers);
	// shut down LDAP
	ldap_close($ds);
}

// That's all Folks!
echo "\nCompleted.\n";

?>